import random
import itertools
from datetime import datetime, timedelta
from tqdm import tqdm  # 用于显示进度条，需要安装：pip install tqdm

class AdvancedIDGenerator:
    def __init__(self):
        self.weights = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        self.check_codes = {0: '1', 1: '0', 2: 'X', 3: '9', 4: '8', 
                          5: '7', 6: '6', 7: '5', 8: '4', 9: '3', 10: '2'}
        self.area_codes = self._load_area_codes()
        self.common_years = list(range(1950, datetime.now().year + 1))
        
    def _load_area_codes(self):
        """加载更全面的行政区划代码"""
        # 这里简化为示例，实际可以从文件或数据库加载完整的行政区划代码
        return {
            '11': ['110101', '110102', '110105'],  # 北京市
            '31': ['310101', '310104', '310105'],  # 上海市
            '44': ['440301', '440304', '440305'],  # 深圳市
            '42': ['420102', '420103', '420104'],  # 武汉市
            '51': ['510104', '510105', '510106']   # 成都市
        }
    
    def _generate_valid_date(self, year_part=None, month_part=None, day_part=None):
        """生成有效日期"""
        if year_part and len(year_part) == 4 and year_part.isdigit():
            year = int(year_part)
        else:
            year = random.choice(self.common_years)
            
        if month_part and month_part.isdigit():
            month = int(month_part)
        else:
            month = random.randint(1, 12)
            
        max_day = 31
        if month in [4, 6, 9, 11]:
            max_day = 30
        elif month == 2:
            max_day = 29 if (year % 400 == 0 or (year % 100 != 0 and year % 4 == 0)) else 28
            
        if day_part and day_part.isdigit():
            day = min(int(day_part), max_day)
        else:
            day = random.randint(1, max_day)
            
        return f"{year:04d}{month:02d}{day:02d}"

    def _smart_fill(self, pattern):
        """智能填充模式中的x"""
        # 处理地区码部分(前6位)
        if pattern[:6] == 'xxxxxx':
            province = random.choice(list(self.area_codes.keys()))
            area_code = random.choice(self.area_codes[province])
        elif pattern[:2] == 'xx':
            province = random.choice(list(self.area_codes.keys()))
            area_code = province + ''.join(random.choice('0123456789') for _ in range(4))
        elif pattern[:6].count('x') > 0:
            fixed_part = pattern[:6].replace('x', '')
            possible_areas = [code for code in self.area_codes.get(pattern[:2], []) 
                             if code.startswith(fixed_part)]
            area_code = random.choice(possible_areas) if possible_areas else ''.join(
                [c if c != 'x' else random.choice('0123456789') for c in pattern[:6]])
        else:
            area_code = pattern[:6]
        
        # 处理出生日期部分(7-14位)
        birth_date = self._generate_valid_date(
            pattern[6:10] if len(pattern) > 10 else None,
            pattern[10:12] if len(pattern) > 12 else None,
            pattern[12:14] if len(pattern) > 14 else None
        )
        
        # 处理顺序码部分(15-17位)
        seq_code = ''.join([c if c != 'x' else random.choice('0123456789') for c in pattern[14:17]])
        
        # 计算校验码
        id_17 = area_code + birth_date + seq_code
        if len(id_17) != 17:
            id_17 = id_17.ljust(17, '0')[:17]
        
        total = sum(int(id_17[i]) * self.weights[i] for i in range(17))
        check_code = self.check_codes[total % 11]
        
        # 处理校验位
        final_check = pattern[-1] if len(pattern) > 17 and pattern[-1] != 'x' else check_code
        
        return id_17 + final_check

    def generate_possible_ids(self, pattern, max_attempts=100000):
        """
        生成所有可能的有效身份证号码
        
        参数:
            pattern: 模糊模式，如"11010519XX1001XXXX"
            max_attempts: 最大尝试次数
            
        返回:
            可能的有效身份证号码列表
        """
        pattern = pattern.upper().replace("X", "x").ljust(18, 'x')[:18]
        valid_ids = []
        
        # 使用进度条显示生成进度
        with tqdm(total=max_attempts, desc="生成中", unit="尝试") as pbar:
            attempts = 0
            while attempts < max_attempts:
                attempts += 1
                candidate = self._smart_fill(pattern)
                if self.validate(candidate) and candidate not in valid_ids:
                    valid_ids.append(candidate)
                pbar.update(1)
        
        return valid_ids

    def validate(self, id_number):
        """验证身份证号码是否有效"""
        if len(id_number) != 18:
            return False
        
        # 验证前17位为数字
        if not id_number[:17].isdigit():
            return False
        
        # 验证出生日期
        try:
            birth_date = datetime.strptime(id_number[6:14], "%Y%m%d")
            if birth_date > datetime.now():
                return False
        except:
            return False
        
        # 验证校验码
        total = sum(int(id_number[i]) * self.weights[i] for i in range(17))
        return self.check_codes[total % 11] == id_number[-1].upper()

    def save_to_file(self, id_list, filename="身份证.txt"):
        """将结果保存到文件"""
        with open(filename, 'w', encoding='utf-8') as f:
            for id_num in id_list:
                f.write(f"{id_num}\n")
                
    def analyze_pattern(self, pattern):
        """分析模式并返回可能的组合数量估算"""
        pattern = pattern.upper().replace("X", "x").ljust(18, 'x')[:18]
        count = 1
        
        # 地区码部分(前6位)
        area_x = pattern[:6].count('x')
        if area_x == 6:
            count *= len(self.area_codes) * 10000  # 估算
        elif area_x > 0:
            count *= 10 ** area_x
        
        # 出生日期部分(7-14位)
        date_part = pattern[6:14]
        date_x = date_part.count('x')
        
        if date_x == 8:
            count *= len(self.common_years) * 12 * 31  # 年*月*日
        elif date_x > 0:
            if date_part[:4].count('x') > 0:  # 年份有x
                count *= len(self.common_years)
            if date_part[4:6].count('x') > 0:  # 月份有x
                count *= 12
            if date_part[6:8].count('x') > 0:  # 日期有x
                count *= 31
        
        # 顺序码部分(15-17位)
        seq_x = pattern[14:17].count('x')
        count *= 10 ** seq_x
        
        # 校验位
        if pattern[-1] == 'x':
            count *= 11
        
        return count

if __name__ == "__main__":
    generator = AdvancedIDGenerator()
    
    print("=== 改进版身份证号码生成器 ===")
    print("说明: 输入18位模式，已知数字填写实际数字，未知部分用x代替")
    print("示例: 1101051990xxxxxx3x 或 xxxxxx20001001xxxxxx")
    
    while True:
        pattern = input("\n请输入模糊身份证模式(或输入q退出): ").strip()
        if pattern.lower() == 'q':
            break
            
        if len(pattern) != 18:
            print("错误: 身份证号码必须为18位!")
            continue
            
        # 分析模式复杂度
        total_possible = generator.analyze_pattern(pattern)
        print(f"模式分析: 可能的组合约{total_possible:,}种")
        
        if total_possible > 100000:
            print("警告: 此模式组合数量过大，生成可能需要较长时间")
            confirm = input("是否继续? (y/n): ")
            if confirm.lower() != 'y':
                continue
                
        possible_ids = generator.generate_possible_ids(pattern)
        
        print("\n生成的有效身份证号码:")
        for i, id_num in enumerate(possible_ids, 1):
            print(f"{i}. {id_num}")
        
        if possible_ids:
            save = input("\n是否保存结果到文件? (y/n): ")
            if save.lower() == 'y':
                filename = input("输入文件名(默认:身份证.txt): ").strip() or "身份证.txt"
                generator.save_to_file(possible_ids, filename)
                print(f"已保存到{filename}")